BASE = "@base"
CONTAINER = "@container"
CONTEXT = "@context"
GRAPH = "@graph"
ID = "@id"
INDEX = "@index"
LANG = "@language"
LIST = u"@list"
REV = u"@reverse"
SET = u"@set"
TYPE = u"@type"
VALUE = u"@value"
VOCAB = u"@vocab"
